##############################################################################
# Tutorial Folder ZClass for Zope, part of the Tutorial Product
#
# Copyright (C) 2001 Dr John A.R. Williams
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

"""Tutorial Product"""

from Products.ZCatalog.ZCatalog import ZCatalog
import Products
import Globals
from Globals import HTMLFile
from DateTime import DateTime
from TutorialTopic import TutorialTopic
from TutorialPath import TutorialPath

manage_addTutorialFolderForm=HTMLFile('addTutorialFolder',globals())

def manage_addTutorialFolder(self, id, title, vocab_id=None, REQUEST=None):
    """Add a Tutorial object
    """
    id=str(id)
    title=str(title)
    vocab_id=str(vocab_id)
    if vocab_id == 'create_default_catalog_':
        vocab_id = None        
    c=TutorialFolder(id, title, vocab_id, self)
    self._setObject(id, c)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class TutorialFolder( ZCatalog ):
  """Tutorial ZClass"""
  
  meta_type = "Tutorial Folder"
  icon="misc_/Tutorial/tutorialfolder"
 
  meta_types=({'permission': 'Manage Tutors Information',
               'name': 'Tutorial Topic',
               'product': 'Tutorial',
               'action': 'manage_addTutorialTopicForm'},
              {'permission': 'Manage Tutors Information',
               'name': 'Tutorial Path',
               'product': 'Tutorial',
               'action': 'manage_addTutorialPathForm'},
              )

  manage_options=ZCatalog.manage_options[0:3]+(
      {'label':'View','action':'index_html',},
      )+ZCatalog.manage_options[3:]

  
  __ac_permissions__=(
    ('Manage Tutors Information',
     ('manage_addTutorialTopicForm','manage_addTutorialTopic',
      'manage_addTutorialPathForm','manage_addTutorialPath',
      'generate_id','newTopic','manage_fix'),),
    ('Access contents information',
     ('topics','topic_titles'),),
    )

  def __init__(self, id, title='',vocab_id=None,container=None):
      ZCatalog.__init__(self,id,title,vocab_id,container)
      self._catalog.addIndex('summary', 'TextIndex')
      self.Catalog=self # I am my own catalog
      self.nextid=1

  manage_addTutorialTopicForm=HTMLFile('addTutorialTopic',globals())

  def manage_addTutorialTopic(self, id, title,
                             summary='',data='',format='html',
                              REQUEST=None):
      """Add a Tutorial Topic"""
      id=str(id)
      c=TutorialTopic(id, title,summary,data,format)
      self._setObject(id, c)
      if REQUEST is not None:
          return self.manage_main(self, REQUEST)

  manage_addTutorialPathForm=HTMLFile('addTutorialPath',globals())

  def manage_addTutorialPath(self, id, title='',summary='',
                             topicids=(),REQUEST=None):
      """Add a Tutorial Path"""
      id=str(id)
      c=TutorialPath(id, title,self._username(),summary,topicids)
      self._setObject(id, c)
      if REQUEST is not None:
          return self.manage_main(self, REQUEST)

  def generate_id(self):
      """Find an available ID"""
      fmt = "%08d"
      while hasattr(self, fmt % self.nextid):
          self.nextid = self.nextid + 1
      return fmt % self.nextid

  def _username(self):
      if hasattr(self,'username'):
          return self.username()
      return self.REQUEST.AUTHENTICATED_USER.getUserName()

  def topics(self):
      return self.objectValues(['Tutorial Topic'])

  def topic_titles(self):
      """returns array of unique topic titles in this tutorial folder"""
      list=[]
      for t in self.topics():
          if t.title not in list:
              list.append(t.title)
      return list
      
  def newTopic(self,title):
      """Create a new empty topic of specified title - return its id"""
      id=self.generate_id()
      c=TutorialTopic(id, title)
      self._setObject(id, c)
      return id

  def manage_fix(self):
      """Routine to fix previous tutorials to most recent design"""
      # remove creator property

      def del_creator(obj):
          if hasattr(obj,'creator'):
              try: delattr(obj,'creator')
              except: None
          if obj.hasProperty('creator'):
              obj.manage_delProperties(('creator',))
          
      for obj in self.topics():
          del_creator(obj)
          for pr in obj.objectValues(['Tutorial Problem']):
              del_creator(obj)
      for obj in self.objectValues(['Tutorial Path']):
          del_creator(obj)

      return "Done"
      
Globals.default__class_init__(TutorialFolder)

