##############################################################################
# Tutorial Path ZClass for Zope, part of the Tutorial Product
#
# Copyright (C) 2001 Dr John A.R. Williams
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

"""Tutorial Product"""

from OFS.Folder import Folder
from Products.ZCatalog.CatalogAwareness import CatalogAware
import Globals
from Globals import HTMLFile
from DateTime import DateTime

class TutorialPath( CatalogAware, Folder ):
  """Tutorial Path Python Class"""

  meta_type = "Tutorial Path"
  icon="misc_/Tutorial/tutorialpath"

  __ac_permissions__=(
    ('Access contents information',
     ('topics','nextTopic','prevTopic','PrincipiaSearchSource'),),
    ('View',
     ('index_html','entire_html'),),
    )

  _properties=({'id':'title', 'type': 'string', 'mode':'w'},
               {'id':'summary', 'type': 'string', 'mode': 'w'},
               {'id':'topicids', 'type': 'lines', 'mode': 'w'},
	       {'id':'created', 'type':'DateTime', 'mode':''}
               )  

  def __init__(self, id, title,summary='',topicids=()):
    self.id=id
    self.title=title
    self.summary=summary
    self.topicids=topicids
    self.created=DateTime()    

  index_html=HTMLFile('viewTutorialPathIndex',globals())

  entire_html=HTMLFile('viewTutorialPathEntire',globals())

  def _updateProperty(self, id, value):
    """Ensure catalog is updated when properties are changed"""
    Folder._updateProperty(self,id,value)
    self.reindex_object()

  def PrincipiaSearchSource(self):
    txt=self.title+self.summary
    if self.topicids:
      for topic in self.topics():
        txt=txt+topic.PrincipiaSearchSource()
    return txt

  def topics(self):
    """return topic objects"""
    topics=[]
    if self.topicids:
      for id in self.topicids:
        topics.append(getattr(self,id))
    return topics

  def _getindex(self,id):
    i=0
    for tid in self.topicids:
      if tid==id: return i
      i=i+1
    return None

  def nextTopic(self,this):
    """returns topic after given one or  else None"""
    i=self._getindex(this.id)
    if (i is not None) and i<len(self.topicids)-1:
      return getattr(self,self.topicids[i+1])

  def prevTopic(self,this):
    """returns topic previous to given one or else None"""
    i=self._getindex(this.id)
    if (i is not None) and i>0:
      return getattr(self,self.topicids[i-1])
   
Globals.default__class_init__(TutorialPath)
