##############################################################################
# Tutorial Topic ZClass for Zope, part of the Tutorial Product
#
# Copyright (C) 2001 Dr John A.R. Williams
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

"""Tutorial Product"""

from OFS.Folder import Folder
from Products.ZCatalog.CatalogAwareness import CatalogAware
import Globals
from Globals import HTMLFile
from  TutorialProblem import TutorialProblem
import re
import string
from urllib import unquote,quote
from DateTime import DateTime
from StructuredText   import html_with_references

class TutorialTopic( CatalogAware, Folder  ):
  """Tutorial Topic Python Class"""

  meta_type = "Tutorial Topic"
  icon="misc_/Tutorial/tutorialtopic"
  
  meta_types=({'permission': 'Manage Tutors Information',
              'name': 'Tutorial Problem',
              'product': 'Tutorial',
              'action': 'manage_addTutorialProblemForm'},
             )

  __ac_permissions__=(
    ('Manage Tutors Information',
     ('manage_addTutorialProblemForm','manage_addTutorialProblem',
      'data_editing'),),
    ('Access contents information',
     ('formats','tutorial','summary','created','data','format',
      'nextTopic','prevTopic','PrincipiaSearchSource','relatedTopics',
      'data_formatted','data_editing'),),
    ('View',('index_html',),),
    )

  _properties=({'id':'title', 'type': 'string', 'mode':'w'},
               {'id':'summary', 'type': 'string', 'mode': 'w'},
               {'id':'data', 'type': 'text', 'mode': 'w'},
               {'id':'format', 'type': 'selection',
                'select_variable':'formats', 'mode': 'w'},
	       {'id':'created', 'type':'DateTime', 'mode':''}
               )

  formats=('html','structured text')

  def __init__(self, id, title,summary='',
               data='',format='html'):
    self.id=id
    self.title=title
    self.summary=summary
    self.data=data
    self.format=format
    self.created=DateTime()

  index_html=HTMLFile('viewTutorialTopic',globals())
  
  manage_addTutorialProblemForm=HTMLFile('addTutorialProblem',globals())

  def manage_addTutorialProblem(self, id, title='',
                                problem='', solution='', REQUEST=None):
      """Add a Tutorial Problem object"""
      id=str(id)
      c=TutorialProblem(id, title,problem,solution)
      self._setObject(id, c)
      if REQUEST is not None:
          return self.manage_main(self, REQUEST)

  def tutorial(self):
    return self.getParentNode()
  
  def _updateProperty(self, id, value):
    """Ensure catalog and folder is updated when properties are changed"""
    if id=='data':
      """Get the content enclosed in braces and then remove the braces"""
      m=re.compile(r"\s{([^\s][^}]+?[^\s])}\s",re.M or re.I)
      links=m.findall(value)
      value=re.sub("[{}]",'',value)
      """Then add topics to then tutorial if not already there"""
      titles=self.tutorial().topic_titles()
      titles.append(self.title)
      for t in links:
        if t not in titles:
          self.tutorial().newTopic(t)
          titles.append(t)
    Folder._updateProperty(self,id,value)
    self.reindex_object()

  def _remapimages(self,data):
    imageid=string.join((self.objectIds(['Image'])+
                         map(lambda id: quote(id),self.objectIds(['Image']))),
                         '|')
    m=re.compile(r"(img.*?src=[\'\"])("+imageid+r".*?)",re.S | re.I)
    return re.sub(m,r"\1"+self.absolute_url()+r"/\2",data)
      
  def data_formatted(self,level=3,absolute=0):
    """Format data into html for presentation"""
    topics=self.tutorial().topics()
    base=self.tutorial().absolute_url()
    data=self.data
    if absolute: data=self._remapimages(data)
    if len(topics)>0:
      topics.sort(_lencmp)
      for t in topics:
        if t.title!=self.title:
          data=re.sub(re.compile("\s(\(*"+t.title+"\)*)\s",re.I),
                      " <a href=\""+base+"/"+t.id+"\">\\1</a> ",
                      data)
    data=re.sub("\r\n","\n",data)
    if self.format=='structured text':
      return str(html_with_references(data, level))
    return data

  def PrincipiaSearchSource(self):
    return self.title+self.summary+self.data
    
  def data_editing(self):
    """Format data for editing"""
    data=re.sub("[{}]","",self.data)
    topics=self.tutorial().topics()
    if len(topics)>0:
      topics.sort(_lencmp)
      pattern="\s("+string.join(map(lambda a: a.title,topics),"|")+")\s"
      return re.sub(re.compile(pattern,re.I|re.M)," {\\1} ",data)
    return data

  def isinPath(self):
    return self.aq_parent.meta_type == 'Tutorial Path'

  def nextTopic(self):
    if hasattr(self.aq_parent,'nextTopic'):
      return self.aq_parent.nextTopic(self)

  def prevTopic(self):
    if hasattr(self.aq_parent,'prevTopic'):
      return self.aq_parent.prevTopic(self)

  def relatedTopics(self):
    catalog=self.tutorial()
    return filter(lambda t,s=self: t!=None and t.id!=s.id,
                  map(lambda r: r.getObject(),
                      catalog.searchResults(
      { 'PrincipiaSearchSource':self.title,
        'meta_type': 'Tutorial Topic'}
      )))
    

def _lencmp(a,b):
  """sorting function - longest title first"""
  ta=len(a.title)
  tb=len(b.title)
  if ta>tb:
    return -1
  return tb<ta

Globals.default__class_init__(TutorialTopic)
